/****************************************************************************
** Meta object code from reading C++ file 'ldapconfigwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kldap-26.07.80/src/widgets/ldapconfigwidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ldapconfigwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12KLDAPWidgets16LdapConfigWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto KLDAPWidgets::LdapConfigWidget::qt_create_metaobjectdata<qt_meta_tag_ZN12KLDAPWidgets16LdapConfigWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KLDAPWidgets::LdapConfigWidget",
        "hostNameChanged",
        "",
        "features",
        "WinFlags",
        "user",
        "bindDn",
        "realm",
        "password",
        "host",
        "port",
        "version",
        "dn",
        "KLDAPCore::LdapDN",
        "filter",
        "mech",
        "security",
        "Security",
        "auth",
        "Auth",
        "sizeLimit",
        "timeLimit",
        "pageSize",
        "WinFlag",
        "W_USER",
        "W_BINDDN",
        "W_REALM",
        "W_PASS",
        "W_HOST",
        "W_PORT",
        "W_VER",
        "W_DN",
        "W_FILTER",
        "W_SECBOX",
        "W_AUTHBOX",
        "W_TIMELIMIT",
        "W_SIZELIMIT",
        "W_PAGESIZE",
        "W_ALL",
        "None",
        "SSL",
        "TLS",
        "Anonymous",
        "Simple",
        "SASL"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'hostNameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'features'
        QtMocHelpers::PropertyData<WinFlags>(3, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'user'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'bindDn'
        QtMocHelpers::PropertyData<QString>(6, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'realm'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'password'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'host'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'port'
        QtMocHelpers::PropertyData<int>(10, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'version'
        QtMocHelpers::PropertyData<int>(11, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'dn'
        QtMocHelpers::PropertyData<KLDAPCore::LdapDN>(12, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'filter'
        QtMocHelpers::PropertyData<QString>(14, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'mech'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'security'
        QtMocHelpers::PropertyData<enum Security>(16, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'auth'
        QtMocHelpers::PropertyData<enum Auth>(18, 0x80000000 | 19, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'sizeLimit'
        QtMocHelpers::PropertyData<int>(20, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'timeLimit'
        QtMocHelpers::PropertyData<int>(21, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'pageSize'
        QtMocHelpers::PropertyData<int>(22, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
        // flag 'WinFlags'
        QtMocHelpers::EnumData<WinFlags>(4, 23, QMC::EnumIsFlag).add({
            {   24, WinFlag::W_USER },
            {   25, WinFlag::W_BINDDN },
            {   26, WinFlag::W_REALM },
            {   27, WinFlag::W_PASS },
            {   28, WinFlag::W_HOST },
            {   29, WinFlag::W_PORT },
            {   30, WinFlag::W_VER },
            {   31, WinFlag::W_DN },
            {   32, WinFlag::W_FILTER },
            {   33, WinFlag::W_SECBOX },
            {   34, WinFlag::W_AUTHBOX },
            {   35, WinFlag::W_TIMELIMIT },
            {   36, WinFlag::W_SIZELIMIT },
            {   37, WinFlag::W_PAGESIZE },
            {   38, WinFlag::W_ALL },
        }),
        // enum 'Security'
        QtMocHelpers::EnumData<enum Security>(17, 17, QMC::EnumFlags{}).add({
            {   39, Security::None },
            {   40, Security::SSL },
            {   41, Security::TLS },
        }),
        // enum 'Auth'
        QtMocHelpers::EnumData<enum Auth>(19, 19, QMC::EnumFlags{}).add({
            {   42, Auth::Anonymous },
            {   43, Auth::Simple },
            {   44, Auth::SASL },
        }),
    };
    return QtMocHelpers::metaObjectData<LdapConfigWidget, qt_meta_tag_ZN12KLDAPWidgets16LdapConfigWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KLDAPWidgets::LdapConfigWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KLDAPWidgets16LdapConfigWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KLDAPWidgets16LdapConfigWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12KLDAPWidgets16LdapConfigWidgetE_t>.metaTypes,
    nullptr
} };

void KLDAPWidgets::LdapConfigWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LdapConfigWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->hostNameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LdapConfigWidget::*)(const QString & )>(_a, &LdapConfigWidget::hostNameChanged, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::assignFlags<WinFlags>(_v, _t->features()); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->user(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->bindDn(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->realm(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->password(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->host(); break;
        case 6: *reinterpret_cast<int*>(_v) = _t->port(); break;
        case 7: *reinterpret_cast<int*>(_v) = _t->version(); break;
        case 8: *reinterpret_cast<KLDAPCore::LdapDN*>(_v) = _t->dn(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->filter(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->mech(); break;
        case 11: *reinterpret_cast<enum Security*>(_v) = _t->security(); break;
        case 12: *reinterpret_cast<enum Auth*>(_v) = _t->auth(); break;
        case 13: *reinterpret_cast<int*>(_v) = _t->sizeLimit(); break;
        case 14: *reinterpret_cast<int*>(_v) = _t->timeLimit(); break;
        case 15: *reinterpret_cast<int*>(_v) = _t->pageSize(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFeatures(*reinterpret_cast<WinFlags*>(_v)); break;
        case 1: _t->setUser(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setBindDn(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setRealm(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setPassword(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setHost(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setPort(*reinterpret_cast<int*>(_v)); break;
        case 7: _t->setVersion(*reinterpret_cast<int*>(_v)); break;
        case 8: _t->setDn(*reinterpret_cast<KLDAPCore::LdapDN*>(_v)); break;
        case 9: _t->setFilter(*reinterpret_cast<QString*>(_v)); break;
        case 10: _t->setMech(*reinterpret_cast<QString*>(_v)); break;
        case 11: _t->setSecurity(*reinterpret_cast<enum Security*>(_v)); break;
        case 12: _t->setAuth(*reinterpret_cast<enum Auth*>(_v)); break;
        case 13: _t->setSizeLimit(*reinterpret_cast<int*>(_v)); break;
        case 14: _t->setTimeLimit(*reinterpret_cast<int*>(_v)); break;
        case 15: _t->setPageSize(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KLDAPWidgets::LdapConfigWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KLDAPWidgets::LdapConfigWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KLDAPWidgets16LdapConfigWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int KLDAPWidgets::LdapConfigWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void KLDAPWidgets::LdapConfigWidget::hostNameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
